<?php if (!defined('PmWiki')) exit();

$filter = getifset($phpinc_params, 'filter', '');
$sort_by = getifset($phpinc_params, 'sort', '');

if (substr($filter, 0, 1) == '$') { $filter = requestvar(substr($filter, 1), ''); }
if (substr($sort_by, 0, 1) == '$') { $sort_by = requestvar(substr($sort_by, 1), ''); }

$filter = preg_replace('![^\w\-\.\*\?]!', '', $filter);

$sort_by = strtolower($sort_by);
switch ($sort_by)
{
  case 'group':
  case 'read':
  case 'edit':
  case 'attr':
    break;
  default:
    $sort_by = 'group';
}

echo "-----\nListing pages named &quot;$filter&quot;, sorted by $sort_by:\n-----\n\n";

$perms = array();
$pages = ListPages($filter);
foreach ($pages as $page) { accpermlist__read_pageinfo($perms, $page, $sort_by); }

$not_sorting_keys = array();
if ($sort_by != 'group') { $not_sorting_keys[] = 'group'; }
if ($sort_by != 'read') { $not_sorting_keys[] = 'read'; }
if ($sort_by != 'edit') { $not_sorting_keys[] = 'edit'; }
if ($sort_by != 'attr') { $not_sorting_keys[] = 'attr'; }

$sortkeys = array_keys($perms);
sort($sortkeys);

$actions = array(
    'group' => '',
    'read' => '',
    'edit' => '(?action=edit)',
    'attr' => '(?action=attr)',
  );

foreach ($sortkeys as $sortkey)
{
  $pages = $perms[$sortkey];

  echo "!!!", ucfirst($sort_by), ': ', $sortkey, "\n||!Name";
  foreach ($not_sorting_keys as $nsk) { echo ' ||!', ucfirst($nsk); }
  echo " ||\n";

  usort($pages, 'accpermlist__pagename_cmp');

  foreach ($pages as $listdata)
  {
    $name = $listdata['name'];
    $namelink = $name;

    if ($sort_by != 'group')
    {
      $namelink = '[[(' . $listdata['group'] . '.)' . $name . $actions[$sort_by] . ']]';
    }

    switch($name)
    {
      case 'GroupAttributes':
        $namelink = '<b>' . $namelink . '</b>'; break;
      case 'RecentChanges':
        $namelink = '<small>' . $namelink . '</small>'; break;
      default: ;
    }

    $page = $listdata['group'] . '.' . $name;
    echo '||' . $namelink;

    foreach($not_sorting_keys as $nsk)
    {
      echo ' ||';
      if ($nsk == 'group')
      {
        echo $listdata[$nsk];
      }
      else
      {
        echo '[[', $page, $actions[$nsk], '|', accpermlist__short_attrs($listdata[$nsk]),
          ' &nbsp;]]';
      }
    }
    echo " ||\n";
  }

  echo "\n";
}


if (isset($phpinc_params['explain']))
{
  echo "-----\n<b>Explanation of Symbols:</b>",
    ' ... = inherited, * = public, - = admins only;',
    ' ~* = any logged in user, ~"name" = only user "name"; $ = password (crypted),',
    ' <i>"nopass" (crypted!)</i> = deprecated, please use <i>@nopass</i> instead;',
    ' ?:text = unknown text', "\n";
}

echo "-----\n\n";


function accpermlist__short_attrs($long)
{
  if ($long == '') { return '...'; }

  $long = explode(' ', $long);
  $short = array();

  foreach ($long as $pw) { $short[] = accpermlist__short_attr($pw); }

  $short = array_unique($short);
  sort($short);
  return implode(', ', $short);
}


function accpermlist__short_attr($long)
{
  if (substr($long, 0, 1) == '@')
  {
    if ($long == '@nopass') { return '<b>*</b>'; }
    if ($long == '@lock') { return '<b>-</b>'; }
    return $long;
  }

  if (substr($long, 0, 3) == 'id:')
  {
    if ($long == 'id:*') { return '<b>~*</b>'; }
    return '~&quot;' . substr($long, 3) . '&quot';
  }

  if (substr($long, 0, 1) == '$')
  {
    if (crypt('nopass', $long) == $long) { return '&quot;nopass&quot; (crypted!)'; };
    return '<b>$</b>';
  }

  return '?:' . $long;
}


function accpermlist__pagename_cmp($a, $b)
{
  // Gruppe geht vor.
  if ($a['group'] != $b['group']) { return strcmp($a['group'], $b['group']); }

  // Sonder-Seiten kommen gleich danach.
  if ($a['name'] == 'GroupAttributes') { return -1; }
  if ($b['name'] == 'GroupAttributes') { return 1; }

  // rest egal.
  return strcmp($a['name'], $b['name']);
}


function accpermlist__read_pageinfo(&$perms, $page, $sort_by)
{
  $pageinfo = ReadPage($page);
  list($group, $name) = explode('.', $page);
  $listdata = array(
      'group' => $group,
      'name' => $name,
      'read' => getifset($pageinfo, 'passwdread', ''),
      'edit' => getifset($pageinfo, 'passwdedit', ''),
      'attr' => getifset($pageinfo, 'passwdattr', ''),
    );

  if ( ($listdata['read'] == '') && ($listdata['edit'] == '') && ($listdata['attr'] == '') )
  {
    return false;  # boring file! has no special permissions set
  }

  $sortkey = $listdata[$sort_by];
  if ($sort_by != 'group')
  {
    $sortkey = accpermlist__short_attrs($sortkey);
    # if ($sortkey == '...') { return false; }  # do not want!
  }

  if (!isset($perms[$sortkey])) { $perms[$sortkey] = array(); }
  $perms[$sortkey][] = $listdata;
  return true;
}